@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 1998, 2002-2006 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
+ * Copyright (c) 1998, 2002-2007 Kiyoshi Matsui <kmatsui@t3.rim.or.jp>
  * All rights reserved.
  *
  * Some parts of this code are derived from the public domain software
@@ -267,15 +267,10 @@ static const SIZES  size_table[] = {
 void    init_eval( void)
 {
     skip = 0;
-#if HAVE_LONG_LONG && COMPILER == INDEPENDENT
-    w_level = 1;
-#else
-    w_level = 2;
-#endif
 }
 #endif
 
-expr_t  eval( void)
+expr_t  eval_if( void)
 /*
  * Evaluate a #if expression.  Straight-forward operator precedence.
  * This is called from directive() on encountering an #if statement.
@@ -302,8 +297,8 @@ expr_t  eval( void)
     skip = skip_cur = opp->skip = 0;        /* Not skipping now     */
 
     while (1) {
-        if (debug & EXPRESSION)
-            fprintf( fp_debug
+        if (mcpp_debug & EXPRESSION)
+            mcpp_fprintf( DBG
                     , "In eval loop skip = %d, binop = %d, line is: %s\n"
                     , opp->skip, binop, infile->bptr);
         skip = opp->skip;
@@ -321,23 +316,23 @@ expr_t  eval( void)
         case OP_FAIL:
             return  0L;                     /* Token error          */
         }
-        if (debug & EXPRESSION)
-            fprintf( fp_debug
+        if (mcpp_debug & EXPRESSION)
+            mcpp_fprintf( DBG
                     , "op = %s, opdope = %04o, binop = %d, skip = %d\n"
                     , opname[ op], opdope[ op], binop, opp->skip);
         if (op == VAL) {                    /* Value?               */
             if (binop != 0) {               /* Binop is needed      */
                 cerror( "Misplaced constant \"%s\""         /* _E_  */
-                        , work, 0L, NULLST);
+                        , work_buf, 0L, NULL);
                 return  0L;
             } else if (& value[ NEXP * 2] <= valp) {
                 cerror( "More than %.0s%ld constants stacked at %s" /* _E_  */
-                        , NULLST, (long) (NEXP * 2 - 1), work);
+                        , NULL, (long) (NEXP * 2 - 1), work_buf);
                 return  0L;
             } else {
-                if (debug & EXPRESSION) {
+                if (mcpp_debug & EXPRESSION) {
                     dump_val( "pushing ", &ev);
-                    fprintf( fp_debug, " onto value stack[%d]\n"
+                    mcpp_fprintf( DBG, " onto value stack[%d]\n"
                             , (int)(valp - value));
                 }
                 valp->val = ev.val;
@@ -350,17 +345,17 @@ expr_t  eval( void)
         if (binop != (prec & 1)) {
             if (op == OP_EOE)
                 cerror( "Unterminated expression"           /* _E_  */
-                        , NULLST, 0L, NULLST);
+                        , NULL, 0L, NULL);
             else
                 cerror( "Operator \"%s\" in incorrect context"      /* _E_  */
-                        , opname[ op], 0L, NULLST);
+                        , opname[ op], 0L, NULL);
             return  0L;
         }
         binop = (prec & 2) >> 1;            /* Binop should follow? */
 
         while (1) {
-            if (debug & EXPRESSION)
-                fprintf( fp_debug
+            if (mcpp_debug & EXPRESSION)
+                mcpp_fprintf( DBG
                         , "op %s, prec %d, stacked op %s, prec %d, skip %d\n"
                 , opname[ op], prec, opname[ opp->op], opp->prec, opp->skip);
 
@@ -372,7 +367,7 @@ expr_t  eval( void)
                             && ++parens == exp_nest_min + 1)
                         cwarn(
                     "More than %.0s%ld nesting of parens"   /* _W4_ */
-                            , NULLST, (long) exp_nest_min, NULLST);
+                            , NULL, (long) exp_nest_min, NULL);
                 } else if (op == OP_QUE) {
                     prec = OP_QUE_PREC;
                 } else if (is_unary( op)) {
@@ -386,7 +381,7 @@ expr_t  eval( void)
                 if (& opstack[ NEXP * 3] <= opp) {
                     cerror(
             "More than %.0s%ld operators and parens stacked at %s"  /* _E_  */
-                            , NULLST, (long) (NEXP * 3 - 1), opname[ op]);
+                            , NULL, (long) (NEXP * 3 - 1), opname[ op]);
                     return  0L;
                 }
                 opp->op = op;
@@ -411,11 +406,11 @@ expr_t  eval( void)
                 } else {                    /* Other operators leave*/
                     opp->skip = op1;        /*  skipping unchanged. */
                 }
-                if (debug & EXPRESSION) {
-                    fprintf( fp_debug, "stacking %s, ", opname[ op]);
+                if (mcpp_debug & EXPRESSION) {
+                    mcpp_fprintf( DBG, "stacking %s, ", opname[ op]);
                     if (&value[0] < valp)
                         dump_val( "valp[-1].val == ", valp - 1);
-                    fprintf( fp_debug, " at %s\n", infile->bptr);
+                    mcpp_fprintf( DBG, " at %s\n", infile->bptr);
                     dump_stack( opstack, opp, value, valp);
                 }
                 break;
@@ -431,7 +426,7 @@ expr_t  eval( void)
             switch ((op1 = opp->op)) {      /* Look at stacked op   */
             case OP_END:                    /* Stack end marker     */
                 if (op == OP_RPA) {         /* No corresponding (   */
-                    cerror( "Excessive \")\"", NULLST, 0L, NULLST); /* _E_  */
+                    cerror( "Excessive \")\"", NULL, 0L, NULL);     /* _E_  */
                     return  0L;
                 }
                 if (op == OP_EOE)
@@ -439,7 +434,7 @@ expr_t  eval( void)
                 break;
             case OP_LPA:                    /* ( on stack           */
                 if (op != OP_RPA) {         /* Matches ) on input?  */
-                    cerror( "Missing \")\"", NULLST, 0L, NULLST);   /* _E_  */
+                    cerror( "Missing \")\"", NULL, 0L, NULL);       /* _E_  */
                     return  0L;
                 }
                 opp--;                      /* Unstack it           */
@@ -452,14 +447,14 @@ expr_t  eval( void)
                 if (opp->op != OP_QUE) {    /* Matches ? on stack?  */
                     cerror(
                     "Misplaced \":\", previous operator is \"%s\""  /* _E_  */
-                            , opname[opp->op], 0L, NULLST);
+                            , opname[opp->op], 0L, NULL);
                     return  0L;
                 }
                 /* Evaluate op1.            Fall through            */
             default:                        /* Others:              */
                 opp--;                      /* Unstack the operator */
-                if (debug & EXPRESSION) {
-                    fprintf( fp_debug, "Stack before evaluation of %s\n"
+                if (mcpp_debug & EXPRESSION) {
+                    mcpp_fprintf( DBG, "Stack before evaluation of %s\n"
                             , opname[ op1]);
                     dump_stack( opstack, opp, value, valp);
                 }
@@ -472,8 +467,8 @@ expr_t  eval( void)
                     return  0L;     /* Out of range or divide by 0  */
                 valp++;
                 skip = 0;
-                if (debug & EXPRESSION) {
-                    fprintf( fp_debug, "Stack after evaluation\n");
+                if (mcpp_debug & EXPRESSION) {
+                    mcpp_fprintf( DBG, "Stack after evaluation\n");
                     dump_stack( opstack, opp, value, valp);
                 }
             }                               /* op1 switch end       */
@@ -489,7 +484,7 @@ expr_t  eval( void)
 
 static int  eval_lex( void)
 /*
- * Return next operator or value to evaluate.  Called from eval().  It calls
+ * Return next operator or value to evaluate.  Called from eval_if().  It calls
  * a special-purpose routines for character constants and numeric values:
  *      eval_char()     called to evaluate 'x'
  *      eval_num()      called to evaluate numbers
@@ -507,7 +502,7 @@ static int  eval_lex( void)
     ev.val = 0L;            /* Default value (on error or 0 value)  */
     c = skip_ws();
     if (c == '\n') {
-        unget();
+        unget_ch();
         return  OP_EOE;                     /* End of expression    */
     }
     token_type = get_unexpandable( c, warn);
@@ -522,26 +517,26 @@ static int  eval_lex( void)
             c1 = c = skip_ws();
             if (c == '(')                   /* Allow defined (name) */
                 c = skip_ws();
-            if (scan_token( c, (workp = work, &workp), work_end) == NAM) {
+            if (scan_token( c, (workp = work_buf, &workp), work_end) == NAM) {
                 if (warn)
                     ev.val = (look_id( identifier) != NULL);
                 if (c1 != '(' || skip_ws() == ')')  /* Balanced ?   */
                     return  VAL;            /* Parsed ok            */
             }
             cerror( "Bad defined syntax: %s"                /* _E_  */
-                    , infile->fp ? "" : infile->buffer, 0L, NULLST);
+                    , infile->fp ? "" : infile->buffer, 0L, NULL);
             break;
-        } else if (cplus) {
+        } else if (cplus_val) {
             if (str_eq( identifier, "true")) {
                 ev.val = 1L;
                 return  VAL;
             } else if (str_eq( identifier, "false")) {
                 ev.val = 0L;
                 return  VAL;
-            } else if (mode != POST_STD
+            } else if (mcpp_mode != POST_STD
                     && (openum = id_operator( identifier)) != 0) {
                 /* Identifier-like operator in C++98    */
-                strcpy( work, identifier);
+                strcpy( work_buf, identifier);
                 return  chk_ops();
             }
         } else if (! standard && str_eq( identifier, "sizeof")) {
@@ -560,30 +555,31 @@ static int  eval_lex( void)
         return  VAL;
     case CHR:                               /* Character constant   */
     case WCHR:                              /* Wide char constant   */
-        if (mode == POST_STD) {
+        if (mcpp_mode == POST_STD) {
             cerror( "Can't use a character constant %s"     /* _E_  */
-                    , work, 0L, NULLST);
+                    , work_buf, 0L, NULL);
             break;
         }
-        valp = eval_char( work);            /* 'valp' points 'ev'   */
+        valp = eval_char( work_buf);        /* 'valp' points 'ev'   */
         if (valp->sign == VAL_ERROR)
             break;
-        if (debug & EXPRESSION) {
+        if (mcpp_debug & EXPRESSION) {
             dump_val( "eval_char returns ", &ev);
-            fputc( '\n', fp_debug);
+            mcpp_fputc( '\n', DBG);
         }
         return  VAL;                        /* Return a value       */
     case STR:                               /* String literal       */
     case WSTR:                              /* Wide string literal  */
-        cerror( "Can't use a string literal %s", work, 0L, NULLST); /* _E_  */
+        cerror(
+    "Can't use a string literal %s", work_buf, 0L, NULL);   /* _E_  */
         break;
     case NUM:                               /* Numbers are harder   */
-        valp = eval_num( work);             /* 'valp' points 'ev'   */
+        valp = eval_num( work_buf);         /* 'valp' points 'ev'   */
         if (valp->sign == VAL_ERROR)
             break;
-        if (debug & EXPRESSION) {
+        if (mcpp_debug & EXPRESSION) {
             dump_val( "eval_num returns ", &ev);
-            fputc( '\n', fp_debug);
+            mcpp_fputc( '\n', DBG);
         }
         return  VAL;
     case OPE:                           /* Operator or punctuator   */
@@ -591,7 +587,7 @@ static int  eval_lex( void)
 
     default:                                /* Total nonsense       */
         cerror( "Can't use the character %.0s0x%02lx"       /* _E_  */
-                , NULLST, (long) c, NULLST);
+                , NULL, (long) c, NULL);
         break;
     }
 
@@ -609,7 +605,7 @@ static int  chk_ops( void)
     case OP_STR:    case OP_CAT:    case OP_ELL:
     case OP_1:      case OP_2:      case OP_3:
         cerror( "Can't use the operator \"%s\""             /* _E_  */
-                , work, 0L, NULLST);
+                , work_buf, 0L, NULL);
         return  OP_FAIL;
     default:
         return  openum;
@@ -699,7 +695,7 @@ static int  do_sizeof( void)
                     typecode &= ~T_INT;
             }
             if ((typecode & ~T_PTR) == 0) {
-                cerror( no_type, NULLST, 0L, NULLST);
+                cerror( no_type, NULL, 0L, NULL);
                 return  OP_FAIL;
             } else {
                 /*
@@ -720,9 +716,9 @@ static int  do_sizeof( void)
         goto  no_good;
     }
 
-    if (debug & EXPRESSION) {
+    if (mcpp_debug & EXPRESSION) {
         if (sizp)
-            fprintf( fp_debug,
+            mcpp_fprintf( DBG,
             "sizp->bits:0x%x sizp->size:0x%x sizp->psize:0x%x ev.val:0x%lx\n"
                     , sizp->bits, sizp->size, sizp->psize
                     , (unsigned long) ev.val);
@@ -730,8 +726,8 @@ static int  do_sizeof( void)
     return  VAL;
 
 no_good:
-    unget();
-    cerror( "sizeof: Syntax error", NULLST, 0L, NULLST);    /* _E_  */
+    unget_ch();
+    cerror( "sizeof: Syntax error", NULL, 0L, NULL);        /* _E_  */
     return  OP_FAIL;
 }
 
@@ -754,17 +750,17 @@ static int  look_type(
         if (token_type == NAM) {
 #if HAVE_LONG_LONG
             if (str_eq( identifier, "long")) {
-                strcpy( work, "long long");
+                strcpy( work_buf, "long long");
                 goto  basic;
             }
 #endif
             if (str_eq( identifier, "double")) {
-                strcpy( work, "long double");
+                strcpy( work_buf, "long double");
                 goto  basic;
             }
         }
-        unget_string( work, NULLST);        /* Not long long        */
-        strcpy( work, "long");              /*   nor long double    */
+        unget_string( work_buf, NULL);      /* Not long long        */
+        strcpy( work_buf, "long");          /*   nor long double    */
     }
 
     /*
@@ -772,30 +768,30 @@ static int  look_type(
      */
 basic:
     for (tp = basic_types; tp->token_name != NULL; tp++) {
-        if (str_eq( work, tp->token_name))
+        if (str_eq( work_buf, tp->token_name))
             break;
     }
 
     if (tp->token_name == NULL) {
         if (! skip) {
-            cerror( unknown_type, work, 0L, NULLST);
+            cerror( unknown_type, work_buf, 0L, NULL);
             return  0;
         } else if (warn_level & 8) {
-            cwarn( unknown_type, work, 0L, non_eval);
+            cwarn( unknown_type, work_buf, 0L, non_eval);
         }
     }
     if ((typecode & tp->excluded) != 0) {
         if (! skip) {
-            cerror( illeg_comb, work, 0L, NULLST);
+            cerror( illeg_comb, work_buf, 0L, NULL);
             return  0;
         } else if (warn_level & 8) {
-            cwarn( illeg_comb, work, 0L, non_eval);
+            cwarn( illeg_comb, work_buf, 0L, non_eval);
         }
     }
 
-    if (debug & EXPRESSION) {
+    if (mcpp_debug & EXPRESSION) {
         if (tp->token_name)
-            fprintf( fp_debug,
+            mcpp_fprintf( DBG,
             "sizeof -- typecode:0x%x tp->token_name:\"%s\" tp->type:0x%x\n"
                     , typecode, tp->token_name, tp->type);
     }
@@ -839,7 +835,7 @@ VAL_SIGN *  eval_num(
 
     ev.sign = SIGNED;                       /* Default signedness   */
     ev.val = 0L;                            /* Default value        */
-    if ((type[ c = *cp++ & UCHARMAX] & DIG) == 0)   /* Dot          */
+    if ((char_type[ c = *cp++ & UCHARMAX] & DIG) == 0)   /* Dot          */
         goto  num_err;
     if (c != '0') {                         /* Decimal              */
         base = 10;
@@ -894,7 +890,7 @@ VAL_SIGN *  eval_num(
                 llflag = TRUE;
                 if (! stdc3 && ((! skip && (warn_level & w_level))
                         || (skip && (warn_level & 8))))
-                    cwarn( ll_suffix, nump, 0L, skip ? non_eval : NULLST);
+                    cwarn( ll_suffix, nump, 0L, skip ? non_eval : NULL);
             } else {
                 lflag = TRUE;
             }
@@ -913,7 +909,7 @@ VAL_SIGN *  eval_num(
         if (c1 == 64) {
             if (! stdc3 && ((! skip && (warn_level & w_level))
                     || (skip && (warn_level & 8))))
-                cwarn( i64_suffix, nump, 0L, skip ? non_eval : NULLST);
+                cwarn( i64_suffix, nump, 0L, skip ? non_eval : NULL);
             cp += 2;
         } else if (c1 == 32 || c1 == 16) {
             cp += 2;
@@ -945,16 +941,16 @@ VAL_SIGN *  eval_num(
 #if HAVE_LONG_LONG
     else if (erange_long && ((skip && (warn_level & 8))
             || (! stdc3 && ! skip && (warn_level & w_level))))
-        cwarn( out_of_range_long, nump, 0L, skip ? non_eval : NULLST);
+        cwarn( out_of_range_long, nump, 0L, skip ? non_eval : NULL);
 #endif
     return  & ev;
 
 range_err:
-    cerror( out_of_range, nump, 0L, NULLST);
+    cerror( out_of_range, nump, 0L, NULL);
     ev.sign = VAL_ERROR;
     return  & ev;
 num_err:
-    cerror( not_integer, nump, 0L, NULLST);
+    cerror( not_integer, nump, 0L, NULL);
     ev.sign = VAL_ERROR;
     return  & ev;
 }
@@ -996,11 +992,11 @@ static VAL_SIGN *   eval_char(
         mbits = CHARBIT * 4;
     else
         mbits = CHARBIT * 2;
-    if (mode == STD && wide) {          /* Wide character constant  */
+    if (mcpp_mode == STD && wide) {          /* Wide character constant  */
         cp++;                           /* Skip 'L'                 */
         bits = mbits;
     }
-    if (type[ *cp & UCHARMAX] & mbstart) {
+    if (char_type[ *cp & UCHARMAX] & mbstart) {
         cl = mb_eval( &cp);
         bits = mbits;
     } else if ((cl = eval_one( &cp, wide, mbits, (ucn8 = FALSE, &ucn8)))
@@ -1012,7 +1008,7 @@ static VAL_SIGN *   eval_char(
     value = cl;
 
     for (i = 0; *cp != '\'' && *cp != EOS; i++) {
-        if (type[ *cp & UCHARMAX] & mbstart) {
+        if (char_type[ *cp & UCHARMAX] & mbstart) {
             cl = mb_eval( &cp);
             if (cl == 0)
                 /* Shift-out sequence of multi-byte or wide character   */
@@ -1040,7 +1036,7 @@ static VAL_SIGN *   eval_char(
                 erange = TRUE;
         }
 #if HAVE_LONG_LONG
-        if ((mode == STD && (! stdc3 && value > ULONGMAX))
+        if ((mcpp_mode == STD && (! stdc3 && value > ULONGMAX))
                 || (! standard && value > LONGMAX))
             erange_long = TRUE;
 #endif
@@ -1058,9 +1054,9 @@ static VAL_SIGN *   eval_char(
     } else if (erange_long && ((skip && (warn_level & 8))
             || (! stdc3 && ! skip && (warn_level & w_level)))) {
         if (wide)
-            cwarn( w_out_of_range_long, token, 0L, skip ? non_eval : NULLST);
+            cwarn( w_out_of_range_long, token, 0L, skip ? non_eval : NULL);
         else
-            cwarn( c_out_of_range_long, token, 0L, skip ? non_eval : NULLST);
+            cwarn( c_out_of_range_long, token, 0L, skip ? non_eval : NULL);
 #endif
     }
 
@@ -1068,22 +1064,22 @@ static VAL_SIGN *   eval_char(
         return  & ev;
 
     if ((! skip && (warn_level & 4)) || (skip && (warn_level & 8))) {
-        if (mode == STD && wide)
+        if (mcpp_mode == STD && wide)
             cwarn(
 "Multi-character wide character constant %s%.0ld%s isn't portable"  /* _W4_ _W8_    */
-                    , token, 0L, skip ? non_eval : NULLST);
+                    , token, 0L, skip ? non_eval : NULL);
         else
             cwarn(
 "Multi-character or multi-byte character constant %s%.0ld%s isn't portable" /* _W4_ _W8_    */
-                    , token, 0L, skip ? non_eval : NULLST);
+                    , token, 0L, skip ? non_eval : NULL);
     }
     return  & ev;
 
 range_err:
     if (wide)
-        cerror( w_out_of_range, token, 0L, NULLST);
+        cerror( w_out_of_range, token, 0L, NULL);
     else
-        cerror( c_out_of_range, token, 0L, NULLST);
+        cerror( c_out_of_range, token, 0L, NULL);
     ev.sign = VAL_ERROR;
     return  & ev;
 }
@@ -1167,7 +1163,7 @@ static expr_t   eval_one(
 
     wchar_max = (UCHARMAX << CHARBIT) | UCHARMAX;
     if (mbits == CHARBIT * 4) {
-        if (mode == STD)
+        if (mcpp_mode == STD)
             wchar_max = (wchar_max << CHARBIT * 2) | wchar_max;
         else
             wchar_max = LONGMAX;
@@ -1216,9 +1212,9 @@ static expr_t   eval_one(
                     && value != 0x24L && value != 0x40L && value != 0x60L)
                 || (!stdc3 && value >= 0xD800L && value <= 0xDFFFL)) {
             if (!skip)
-                cerror( ucn_malval, NULLST, (long) value, NULLST);
+                cerror( ucn_malval, NULL, (long) value, NULL);
             else if (warn_level & 8)
-                cwarn( ucn_malval, NULLST, (long) value, NULLST);
+                cwarn( ucn_malval, NULL, (long) value, NULL);
         }
         if (count >= 8 && uc1 == 'U')
             *ucn8 = TRUE;
@@ -1237,7 +1233,7 @@ undefined:
     if ((! skip && (warn_level & 1)) || (skip && (warn_level & 8)))
         cwarn(
     "Undefined escape sequence%s %.0ld'%s'"         /* _W1_ _W8_    */
-                , skip ? non_eval : NULLST, 0L, seq);
+                , skip ? non_eval : NULL, 0L, seq);
     **seq_pp = uc1;
     *seq_pp = seq + 1;
     return  (expr_t) '\\';              /* Returns the escape char  */
@@ -1247,12 +1243,12 @@ range_err:
     **seq_pp = EOS;                         /* For diagnostic       */
     if (wide) {
         if (! skip)
-            cerror( out_of_range, NULLST, (long) mbits, seq);
+            cerror( out_of_range, NULL, (long) mbits, seq);
         else if (warn_level & 8)
             cwarn( out_of_range, non_eval, (long) mbits, seq);
     } else {
         if (! skip)
-            cerror( out_of_range, NULLST, (long) CHARBIT, seq);
+            cerror( out_of_range, NULL, (long) CHARBIT, seq);
         else if (warn_level & 8)
             cwarn( out_of_range, non_eval, (long) CHARBIT, seq);
     }
@@ -1295,13 +1291,13 @@ static VAL_SIGN *   eval_eval(
     }
     v1 = (--valp)->val;
     sign1 = valp->sign;
-    if (debug & EXPRESSION) {
-        fprintf( fp_debug, "%s op %s", (is_binary( op)) ? "binary" : "unary"
+    if (mcpp_debug & EXPRESSION) {
+        mcpp_fprintf( DBG, "%s op %s", (is_binary( op)) ? "binary" : "unary"
                 , opname[ op]);
         dump_val( ", v1 = ", valp);
         if (is_binary( op))
             dump_val( ", v2 = ", valp + 1);
-        fputc( '\n', fp_debug);
+        mcpp_fputc( '\n', DBG);
     }
 
     if (standard
@@ -1315,7 +1311,7 @@ static VAL_SIGN *   eval_eval(
 
             v3 = (sign1 == SIGNED ? v1 : v2);
             sprintf( negate, neg_format, v3, v3);
-            cwarn( negate, skip ? non_eval : NULLST, 0L, NULLST);
+            cwarn( negate, skip ? non_eval : NULL, 0L, NULL);
         }
         valp->sign = sign1 = sign2 = UNSIGNED;
     }
@@ -1323,24 +1319,24 @@ static VAL_SIGN *   eval_eval(
             && ((! skip && (warn_level & 1)) || (skip && (warn_level & 8)))) {
         if (v2 < 0L || v2 >= sizeof (expr_t) * CHARBIT)
             cwarn( "Illegal shift count %.0s\"%ld\"%s"      /* _W1_ _W8_    */
-                , NULLST, (long) v2, skip ? non_eval : NULLST);
+                , NULL, (long) v2, skip ? non_eval : NULL);
 #if HAVE_LONG_LONG
         else if (! stdc3 && v2 >= sizeof (long) * CHARBIT
                 && ((! skip && (warn_level & w_level))
                     || (skip && (warn_level & 8))))
             cwarn(
 "Shift count %.0s\"%ld\" is larger than bit count of long%s"    /* _W1_ _W8_*/
-                , NULLST, (long) v2, skip ? non_eval : NULLST);
+                , NULL, (long) v2, skip ? non_eval : NULL);
 #endif
     }
     if ((op == OP_DIV || op == OP_MOD) && v2 == 0L) {
         if (! skip) {
-            cerror( zero_div, NULLST, 0L, NULLST);
+            cerror( zero_div, NULL, 0L, NULL);
             valp->sign = VAL_ERROR;
             return  valp;
         } else {
             if (warn_level & 8)
-                cwarn( zero_div, NULLST, 0L, non_eval);
+                cwarn( zero_div, NULL, 0L, non_eval);
             valp->sign = sign1;
             valp->val = (expr_t) EXPR_MAX;
             return  valp;
@@ -1431,7 +1427,7 @@ static expr_t   eval_signed(
         else if (! stdc3 && (v1 < 0L || v2 < 0L)
                 && ((! skip && (warn_level & 1))
                     || (skip && (warn_level & 8))))
-            cwarn( not_portable, op_name, 0L, skip ? non_eval : NULLST);
+            cwarn( not_portable, op_name, 0L, skip ? non_eval : NULL);
         if (op == OP_DIV)
             v1 /= v2;
         else
@@ -1468,7 +1464,7 @@ static expr_t   eval_signed(
         if (v1 < 0L
                 && ((!skip && (warn_level & 1))
                     || (skip && (warn_level & 8))))
-            cwarn( not_portable, op_name, 0L, skip ? non_eval : NULLST);
+            cwarn( not_portable, op_name, 0L, skip ? non_eval : NULL);
         v1 >>= v2;
         break;
     case OP_LT:     v1 = (v1 < v2);     break;
@@ -1490,7 +1486,7 @@ static expr_t   eval_signed(
         v1 = (--*valpp)->val ? v1 : v2;
         break;
     default:
-        cfatal( illeg_op, op_name, 0L, NULLST);
+        cfatal( illeg_op, op_name, 0L, NULL);
     }
 
     *valpp = valp;
@@ -1585,7 +1581,7 @@ static expr_t   eval_unsigned(
             v1 = v2u;
         break;
     default:
-        cfatal( illeg_op, op_name, 0L, NULLST);
+        cfatal( illeg_op, op_name, 0L, NULL);
     }
 
     *valpp = valp;
@@ -1614,9 +1610,9 @@ static void overflow(
         /* Else don't warn  */
     } else if (standard && (*valpp)->sign == UNSIGNED) {/* Never overflow   */
         if (warn_level & 1)
-            cwarn( out_of_range, op_name, 0L, NULLST);
+            cwarn( out_of_range, op_name, 0L, NULL);
     } else {
-        cerror( out_of_range, op_name, 0L, NULLST);
+        cerror( out_of_range, op_name, 0L, NULL);
         (*valpp)->sign = VAL_ERROR;
     }
 }
@@ -1637,7 +1633,7 @@ static void dump_val(
 #endif
     int     sign = valp->sign;
 
-    fprintf( fp_debug, format, msg, sign ? "" : "un", valp->val);
+    mcpp_fprintf( DBG, format, msg, sign ? "" : "un", valp->val);
 }
 
 static void dump_stack(
@@ -1651,19 +1647,19 @@ static void dump_stack(
  */
 {
     if (opstack < opp)
-        fprintf( fp_debug, "Index op prec skip name -- op stack at %s"
+        mcpp_fprintf( DBG, "Index op prec skip name -- op stack at %s"
                 , infile->bptr);
 
     while (opstack < opp) {
-        fprintf( fp_debug, " [%2d] %2d %04o    %d %s\n", (int)(opp - opstack)
+        mcpp_fprintf( DBG, " [%2d] %2d %04o    %d %s\n", (int)(opp - opstack)
                 , opp->op, opp->prec, opp->skip, opname[ opp->op]);
         opp--;
     }
 
     while (value <= --valp) {
-        fprintf( fp_debug, "value[%d].val = ", (int)(valp - value));
+        mcpp_fprintf( DBG, "value[%d].val = ", (int)(valp - value));
         dump_val( "", valp);
-        fputc( '\n', fp_debug);
+        mcpp_fputc( '\n', DBG);
     }
 }
 
